==> simple_billing-0.0.9.tar.gz: lib/simple_billing.dart <==
library simple_billing;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:hive/hive.dart';
import 'package:hive_flutter/hive_flutter.dart';
import 'package:path_provider/path_provider.dart' as path_provider;

part 'local_data.dart';
part 'pop_up_widget.dart';

class SimpleBilling {
  static Set<PurchasedItem> _purchases = {};
  static List<IAPItem> _productItems;
  static List<IAPItem> _subscriptionItems;
  static List<String> _productsId;
  static List<String> _subscriptionsId;
  static String _purchaseTokenAndroid;
  static final Set<String> _consumables = {};
  static StreamSubscription _purchaseUpdatedSubscription;
  static StreamSubscription<String> _purchasePromotedSubscription;
  static StreamSubscription _purchaseErrorSubscription;
  static BuildContext _context;
  static Widget _popUpWidget;
  static void Function(PurchasedItem purchasedItem) _onPurchaseUpdated;

  static Future init({
    List<String> productsId,
    List<String> subscriptionsId,
    String purchaseTokenAndroid,
    Widget popUpWidget = const _DefaultPopUpWidget(),
    void Function(PurchasedItem purchasedItem) onPurchaseUpdated,
  }) async {
    _onPurchaseUpdated = onPurchaseUpdated;
    _popUpWidget = popUpWidget;
    _productsId = productsId;
    _subscriptionsId = subscriptionsId;
    _purchaseTokenAndroid = purchaseTokenAndroid;
    await _SimpleBillingData.init();
    await FlutterInappPurchase.instance.initConnection;
    _purchaseErrorListen();
    _purchasePromotedListen();
    _purchaseUpdateListen();
    await _consumeAllItems();
    await _getPurchases();
    await _getProducts();
    await _getSubscriptions();
  }

  //! Getters

  static List<PurchasedItem> get purchases => _purchases.toList();

  static IAPItem getIAPItem(String productId) => _SimpleBillingData.allIAPItems
      .firstWhere((e) => e.productId == productId, orElse: () => null);

  static List<IAPItem> get allProducts => _SimpleBillingData.productItems;

  static List<IAPItem> get allSubscriptions =>
      _SimpleBillingData.subscriptionItems;

  static String getPrice(String productId,
      {bool withIntroductoryPrice = false}) {
    final IAPItem item = getIAPItem(productId);
    String introductoryPrice;
    if (withIntroductoryPrice) {
      introductoryPrice = item?.introductoryPrice;
    }
    return introductoryPrice ?? item?.localizedPrice;
  }

  static DiscountIOS getDiscountIOS(
      {@required String productId, @required String discountId}) {
    final IAPItem item = getIAPItem(productId);
    return item?.discountsIOS
        ?.firstWhere((e) => e.identifier == discountId, orElse: () => null);
  }

  //! Buy

  static Future buyProduct(String productId,
      {@required BuildContext context, bool consumable = false}) async {
    if (consumable) _consumables.add(productId);
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance.requestPurchase(
        productId,
        purchaseTokenAndroid: _purchaseTokenAndroid,
      );
    } catch (e) {
      _closeLoading();
      print(e);
    }
  }

  /// Request a subscription on `Android` or `iOS`.
  /// Result will be received
  /// in `purchaseUpdated` listener or `purchaseError` listener.
  ///
  /// **NOTICE** second parameter is required on `Android`.
  ///
  /// Identical to [requestPurchase] on `iOS`.
  static Future buySubscription(String subscriptionId,
      {@required BuildContext context, String oldSkuAndroid}) async {
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance.requestSubscription(subscriptionId,
          oldSkuAndroid: oldSkuAndroid,
          purchaseTokenAndroid: _purchaseTokenAndroid);
    } catch (e) {
      // _closeLoading();
      print(e);
    }
  }

  static Future buyProductWithOfferIOS({
    @required String subscriptionId,
    @required String forUser,
    @required Map<String, dynamic> withOffer,
    @required BuildContext context,
  }) async {
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance
          .requestProductWithOfferIOS(subscriptionId, forUser, withOffer);
    } catch (e) {
      _closeLoading();
      print(e);
    }
  }

  static Future<void> restorePurchases({@required BuildContext context}) async {
    _openLoading(context);
    try {
      await _getProducts();
    } finally {
      _closeLoading();
    }
  }

  //! Checks

  static bool checkPurchase(String productId) {
    return purchases.firstWhere((e) => e.productId == productId,
            orElse: () => null) !=
        null;
  }

  static bool checkIfHaveAnyPurchase(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a || b);

  static bool checkIfEverythingIsPurchased(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a && b);

  //! Privates

  static Future _getPurchases() async {
    List<PurchasedItem> items;
    try {
      items = await FlutterInappPurchase.instance.getAvailablePurchases();
    } catch (e) {
      print(e);
    } finally {
      if (items != null) {
        _purchases = items.toSet();
        await _SimpleBillingData.savePurchases(_purchases);
      } else {
        _purchases = _SimpleBillingData.purchasedItems;
      }
      for (final purchase in _purchases) {
        if (_consumables.contains(purchase.productId)) {
          try {
            FlutterInappPurchase.instance
                .consumePurchaseAndroid(purchase.purchaseToken);
          } catch (e) {
            print(e);
          }
        }
        _onPurchaseUpdated(purchase);
      }
    }
  }

  static Future _consumeAllItems() async {
    try {
      await FlutterInappPurchase.instance.consumeAllItems;
    } catch (e) {
      print(e);
    }
  }

  static Future _getProducts() async {
    try {
      _productItems =
          await FlutterInappPurchase.instance.getProducts(_productsId);
      await _SimpleBillingData.saveProductsItem(_productItems);
    } catch (e) {
      _productItems = _SimpleBillingData.productItems;
    }
  }

  static Future _getSubscriptions() async {
    try {
      _subscriptionItems = await FlutterInappPurchase.instance
          .getSubscriptions(_subscriptionsId);
      await _SimpleBillingData.saveSubscriptionsItem(_subscriptionItems);
    } catch (e) {
      _subscriptionItems = _SimpleBillingData.subscriptionItems;
    }
  }

  //! Listen's

  static void _purchaseErrorListen() {
    _purchaseErrorSubscription = FlutterInappPurchase.purchaseError
        .listen((PurchaseResult purchaseError) {
      _closeLoading();
    });
  }

  static void _purchasePromotedListen() {
    _purchasePromotedSubscription =
        FlutterInappPurchase.purchasePromoted.listen(
      (String onData) {},
    );
  }

  static void _purchaseUpdateListen() {
    _purchaseUpdatedSubscription = FlutterInappPurchase.purchaseUpdated
        .listen((PurchasedItem purchase) async {
      _closeLoading();
      await _getProducts();
      if (_consumables.contains(purchase.productId)) {
        await FlutterInappPurchase.instance
            .consumePurchaseAndroid(purchase.purchaseToken);
      }
      _onPurchaseUpdated(purchase);
    });
  }

  //! Dispose

  static Future<void> dispose() async {
    await FlutterInappPurchase.instance.endConnection;
    _purchaseUpdatedSubscription?.cancel();
    _purchaseErrorSubscription?.cancel();
    _purchasePromotedSubscription?.cancel();
  }

  //! Loading

  static void _closeLoading() {
    if (_context != null && _popUpWidget != null) {
      Navigator.pop(_context);
      _context = null;
    }
  }

  static void _openLoading(BuildContext context) {
    _context = context;
    if (_context != null && _popUpWidget != null) {
      showDialog(context: context, builder: (context) => _popUpWidget);
    }
  }
}

==> simple_billing-0.0.11.tar.gz: lib/simple_billing.dart <==
library simple_billing;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:hive/hive.dart';
import 'package:hive_flutter/hive_flutter.dart';
import 'package:path_provider/path_provider.dart' as path_provider;

part 'local_data.dart';
part 'pop_up_widget.dart';

class SimpleBilling {
  static Set<PurchasedItem> _purchases = {};
  static List<IAPItem> _productItems;
  static List<IAPItem> _subscriptionItems;
  static List<String> _productsId;
  static List<String> _consumablesId;
  static List<String> _subscriptionsId;
  static String _purchaseTokenAndroid;
  static StreamSubscription _purchaseUpdatedSubscription;
  static StreamSubscription<String> _purchasePromotedSubscription;
  static StreamSubscription _purchaseErrorSubscription;
  static BuildContext _context;
  static Widget _popUpWidget;
  static void Function(PurchasedItem purchasedItem) _onPurchaseUpdated;

  static Future init({
    @required List<String> productsId,
    @required List<String> consumablesId,
    @required List<String> subscriptionsId,
    String purchaseTokenAndroid,
    Widget popUpWidget = const _DefaultPopUpWidget(),
    void Function(PurchasedItem purchasedItem) onPurchaseUpdated,
  }) async {
    _onPurchaseUpdated = onPurchaseUpdated;
    _popUpWidget = popUpWidget;
    _productsId = productsId;
    _consumablesId = consumablesId;
    _subscriptionsId = subscriptionsId;
    _purchaseTokenAndroid = purchaseTokenAndroid;
    await _SimpleBillingData.init();
    await FlutterInappPurchase.instance.initConnection;
    _purchaseErrorListen();
    _purchasePromotedListen();
    _purchaseUpdateListen();
    await _getPurchases();
    await _getProducts();
    await _getSubscriptions();
  }

  //! Getters

  static List<PurchasedItem> get purchases => _purchases.toList();

  static IAPItem getIAPItem(String productId) => _SimpleBillingData.allIAPItems
      .firstWhere((e) => e.productId == productId, orElse: () => null);

  static List<IAPItem> get allProducts => _SimpleBillingData.productItems;

  static List<IAPItem> get allSubscriptions =>
      _SimpleBillingData.subscriptionItems;

  static String getPrice(String productId,
      {bool withIntroductoryPrice = false}) {
    final IAPItem item = getIAPItem(productId);
    String introductoryPrice;
    if (withIntroductoryPrice) {
      introductoryPrice = item?.introductoryPrice;
    }
    return introductoryPrice ?? item?.localizedPrice;
  }

  static DiscountIOS getDiscountIOS(
      {@required String productId, @required String discountId}) {
    final IAPItem item = getIAPItem(productId);
    return item?.discountsIOS
        ?.firstWhere((e) => e.identifier == discountId, orElse: () => null);
  }

  //! Buy

  static Future buyProduct(String productId,
      {@required BuildContext context}) async {
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance.requestPurchase(
        productId,
        purchaseTokenAndroid: _purchaseTokenAndroid,
      );
    } catch (e) {
      _closeLoading();
      print(e);
    }
  }

  /// Request a subscription on `Android` or `iOS`.
  /// Result will be received
  /// in `purchaseUpdated` listener or `purchaseError` listener.
  ///
  /// **NOTICE** second parameter is required on `Android`.
  ///
  /// Identical to [requestPurchase] on `iOS`.
  static Future buySubscription(String subscriptionId,
      {@required BuildContext context, String oldSkuAndroid}) async {
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance.requestSubscription(subscriptionId,
          oldSkuAndroid: oldSkuAndroid,
          purchaseTokenAndroid: _purchaseTokenAndroid);
    } catch (e) {
      // _closeLoading();
      print(e);
    }
  }

  static Future buyProductWithOfferIOS({
    @required String subscriptionId,
    @required String forUser,
    @required Map<String, dynamic> withOffer,
    @required BuildContext context,
  }) async {
    try {
      _openLoading(context);
      await FlutterInappPurchase.instance
          .requestProductWithOfferIOS(subscriptionId, forUser, withOffer);
    } catch (e) {
      _closeLoading();
      print(e);
    }
  }

  static Future<void> restorePurchases({@required BuildContext context}) async {
    _openLoading(context);
    try {
      await _getPurchases();
    } finally {
      _closeLoading();
    }
  }

  //! Checks

  static bool checkPurchase(String productId) {
    return purchases.firstWhere((e) => e.productId == productId,
            orElse: () => null) !=
        null;
  }

  static bool checkIfHaveAnyPurchase(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a || b);

  static bool checkIfEverythingIsPurchased(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a && b);

  //! Privates

  static Future _getPurchases() async {
    List<PurchasedItem> items;
    try {
      items = await FlutterInappPurchase.instance.getAvailablePurchases();
    } catch (e) {
      print(e);
    } finally {
      if (items != null) {
        _purchases = items.toSet();
        await _SimpleBillingData.savePurchases(_purchases);
      } else {
        _purchases = _SimpleBillingData.purchasedItems;
      }
      for (final purchase in _purchases) {
        _onPurchaseUpdated(purchase);
      }
    }
  }

  static Future _getProducts() async {
    try {
      _productItems = await FlutterInappPurchase.instance
          .getProducts([..._productsId, ..._consumablesId]);
      await _SimpleBillingData.saveProductsItem(_productItems);
    } catch (e) {
      _productItems = _SimpleBillingData.productItems;
    }
  }

  static Future _getSubscriptions() async {
    try {
      _subscriptionItems = await FlutterInappPurchase.instance
          .getSubscriptions(_subscriptionsId);
      await _SimpleBillingData.saveSubscriptionsItem(_subscriptionItems);
    } catch (e) {
      _subscriptionItems = _SimpleBillingData.subscriptionItems;
    }
  }

  //! Listen's

  static void _purchaseErrorListen() {
    _purchaseErrorSubscription = FlutterInappPurchase.purchaseError
        .listen((PurchaseResult purchaseError) {
      _closeLoading();
    });
  }

  static void _purchasePromotedListen() {
    _purchasePromotedSubscription =
        FlutterInappPurchase.purchasePromoted.listen(
      (String onData) {},
    );
  }

  static void _purchaseUpdateListen() {
    _purchaseUpdatedSubscription = FlutterInappPurchase.purchaseUpdated
        .listen((PurchasedItem purchase) async {
      _closeLoading();
      _purchases?.add(purchase);
      await _SimpleBillingData.savePurchases(_purchases);
      await FlutterInappPurchase.instance.finishTransaction(purchase,
          isConsumable: _consumablesId.contains(purchase.productId));
      _onPurchaseUpdated(purchase);
    });
  }

  //! Dispose

  static Future<void> dispose() async {
    await FlutterInappPurchase.instance.endConnection;
    _purchaseUpdatedSubscription?.cancel();
    _purchaseErrorSubscription?.cancel();
    _purchasePromotedSubscription?.cancel();
  }

  //! Loading

  static void _closeLoading() {
    if (_context != null && _popUpWidget != null) {
      Navigator.pop(_context);
      _context = null;
    }
  }

  static void _openLoading(BuildContext context) {
    _context = context;
    if (_context != null && _popUpWidget != null) {
      showDialog(context: context, builder: (context) => _popUpWidget);
    }
  }
}
